%PLOT_DPENDULUM
% Function which geneates a selection of plots which illustrate the
% dynamics of the Double Pendulum
%
% LAST UPDATED by Andy French 10-Dec-2005

function plot_dpendulum(plot_a,plot_xyt,plot_xy,plot_png,plot_ke_pe,...
    plot_j,plot_adot_addot,plot_xydot_xyddot,plot_speed,plot_poincare,...
    t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2,...
    m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init,num_frames)

%Plot and save frames of pendulum animation. Save TIFF files to a directory
%/Animation
if plot_png==1
    
    %If directory /animation already exists, delete it!
    if exist([pwd,'\Animation'])
        disp('Deleting old animation directory ....')
        rmdir([pwd,'\Animation'],'s')
    end
    
    %Create animation directory
    animdir=[pwd,'\Animation\1-99'];
    mkdir(animdir);
    
    %Compute frame times
    t_frame=linspace(0,T,num_frames);
    
    %Generate coordinates of a unit sphere
    [xs,ys,zs]=sphere(100);
    
    %Generate coordinates of a unit circle
    [xc,yc]=circle(200);
    
    %Scale sphere
    xs=xs*0.3;
    ys=ys*0.3;
    zs=zs*0.3;
    
    %Generate figure and axes
    figure('color',[1 1 1],'name','Double Pendulum',...
        'units','pixels','numbertitle','off')
    screendim=get(0,'screensize');
    set(gcf,'position',[screendim(3)/2-250,screendim(4)/2-250,500,500]);
    
    %Compute indices of t which are nearest to frame times and use this to
    %plot and save frames
    for n=1:num_frames
        pause(0.1)
        ti=approx_find(t,t_frame(n));
        set(gcf,'name',['Double Pendulum frame ',num2str(n),' \ ',num2str(num_frames)]);
        set(gca,'nextplot','replace')
        
        %Plot black sphere fixed point and allow other objects to be added
        %to axes
        fp=surf(xs,ys,zs);
        set(fp,'facecolor',[0 0 0],'edgecolor','none');
        hold on
        
        %Plot blue sphere for bob 1
        bob1=surf(xs + x1(ti),ys +y1(ti), zs);
        set(bob1,'facecolor',[0 0 1],'edgecolor','none');
        
        %Plot red sphere for bob 2
        bob2=surf(xs + x2(ti),ys +y2(ti), zs);
        set(bob2,'facecolor',[1 0 0],'edgecolor','none');
        
        %Plot lines joining fixed point to bobs 1 and 2
        plot([0,x1(ti),x2(ti)],[0,y1(ti),y2(ti)],'k')
        
        %Plot circles defining possible positions of bobs 1 and 2
        circle1=plot(xc*l1,yc*l1);
        set(circle1,'color',[0,0,1]);
        circle2=plot(xc*l2 + x1(ti),yc*l2 + y1(ti));
        set(circle2,'color',[1,0,0]);
        
        %Plot trajectory of bob 2
        plot(x2(1:ti),y2(1:ti),'g')
        
        %Set 2D view and axis limits
        view(2)
        xlim([-l1-2*l2,l1+2*l2])
        ylim([-l1-2*l2,l1+2*l2])
        zlim([-l1-2*l2,l1+2*l2])
        axis off
        camlight
        lighting phong
        
        %Set title, with time information
        title({'\bfDouble pendulum',...
            ['\rmm1=',num2str(m1),'kg  m2=',num2str(m2),...
            'kg  l1=',num2str(l1),' metres  l2=',num2str(l2),' metres'],...
            ['time =',num2str(floor(t(ti))),' s']});
        
        %If n exceeds an integer multiple of 100, create a new directory
        %Save indexed png frame to directory /animation
        if (n/100)==round(n/100)
            animdir=[pwd,'\Animation\',num2str(n),' to ',num2str(n+99)];
            mkdir(animdir)
        end
        fig2pic(gcf,'png',['frame',leadingzero(n,3)],animdir);
    end
    disp('New animation frames written to disk!')
end

%%

%Plot bob rod angles from the vertical /degrees vs time /seconds
if plot_a==1
    figure('color',[1 1 1],'name','Double Pendulum bob rod angles')
    subplot(2,1,1)
    plot(t,a1)
    xlabel('Time /s')
    ylabel('Angle /deg')
    title('Pendulum bob rod 1 angle clockwise from vertical')
    subplot(2,1,2)
    plot(t,a2)
    xlabel('Time /s')
    ylabel('Angle /deg')
    title('Pendulum bob rod 2 angle clockwise from vertical')
end

%%

%Plot bob Cartesian X and Y coordinates /metres vs time /s
if plot_xyt==1
    figure('color',[1 1 1],'name','Double Pendulum bob X and Y positions')
    subplot(2,1,1)
    plot(t,x1,t,x2)
    xlabel('Time /s')
    ylabel('x /metres')
    legend('bob1','bob2')
    title('Pendulum bob X positions')
    subplot(2,1,2)
    plot(t,y1,t,y2)
    xlabel('Time /s')
    ylabel('y /metres')
    legend('bob1','bob2')
    title('Pendulum bob Y positions')   
end

%%

%Plot Cartesian X,Y trajectories of pendulum bobs
if plot_xy==1
    figure('color',[1 1 1],'name','Double Pendulum bob X,Y trajectories')
    p1=plot(x1,y1,'go');
    set(p1,'markersize',1,'markerfacecolor',[0,1,0])
    hold on
    p2=plot(x2,y2,'ko');
    set(p2,'markersize',1,'markerfacecolor',[0,0,0])
    xlabel('x /metres')
    ylabel('y /metres')
    title('Pendulum bob X,Y trajectories')
    tl=thelegend({'bob1','bob2'},{'g-','k-'});
    axis equal
end

%%

%Plot kinetic and potential energies
if plot_ke_pe==1
    figure('color',[1 1 1],'name','Double Pendulum kinetic and potential energies')
    subplot(2,2,1);
    plot(t,ke1,t,pe1)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Pendulum bob 1 kinetic, potential energy')
    legend('Kinetic','Potential')
    axis tight
    subplot(2,2,2);
    plot(t,ke2,t,pe2)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Pendulum bob 2 kinetic, potential energy')
    legend('Kinetic','Potential')   
    axis tight
    subplot(2,2,3);
    plot(t,ke1+pe1,t,ke2+pe2)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Total energies, per bob')
    legend('Bob 1','Bob 2') 
    axis tight
    subplot(2,2,4);
    [E,A,B,C]=total_energy(a1*pi/180,a2*pi/180,...
        adot1*pi/180,adot2*pi/180,m1,m2,l1,l2,g);
    plot(t,ke1+pe1+ke2+pe2,t,E)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Total energy of system')
    legend('Solver outputs','total\_energy.m')
    axis tight
end

%%

%Plot angular momentum projected onto Z axis
if plot_j==1
    figure('color',[1 1 1],'name','Double Pendulum angular momenta')
    plot(t,j1,t,j2,t,j1+j2)
    xlabel('Time /s')
    ylabel('Angular momentum /kgm^2s^{-1}')
    title('Angular momentum')
    legend('Bob 1','Bob 2','Total')
end

%%

%Plot angular velocity and angular acceleration
if plot_adot_addot==1
    figure('color',[1 1 1],'name','Double Pendulum angular velocities and angular accelerations')
    subplot(2,1,1);
    plot(t,adot1,t,adot2)
    xlabel('Time /s')
    ylabel('Angular velocities / deg s^{-1}')
    title('Angular velocities')
    legend('Bob 1','Bob 2')
    subplot(2,1,2);
    plot(t,addot1,t,addot2)
    xlabel('Time /s')
    ylabel('Angular acceleration /deg s^{-2}')
    title('Angular acceleration')
    legend('Bob 1','Bob 2')  
end

%%

%Plot Cartesian velocities /ms^-1 and accelerations /ms^-2
if plot_xydot_xyddot==1
    figure('color',[1 1 1],'name',...
    'Double Pendulum X and Y velocities and accelerations')
    subplot(2,2,1)
    plot(t,xdot1,t,ydot1)
    xlabel('Time /s')
    ylabel('Velocity /ms^{-1}')
    title('Pendulum bob 1 X and Y velocities')
    axis tight
    subplot(2,2,2)
    plot(t,xddot1,t,yddot1)
    xlabel('Time /s')
    ylabel('Acceleration /ms^{-2}')
    title('Pendulum bob 2 X and Y acceleration')
    axis tight
    subplot(2,2,3)
    plot(t,xdot2,t,ydot2)
    xlabel('Time /s')
    ylabel('Velocity /ms^{-1}')
    title('Pendulum bob 1 X and Y velocities')
    axis tight
    subplot(2,2,4)
    plot(t,xddot2,t,yddot2)
    xlabel('Time /s')
    ylabel('Acceleration /ms^{-2}')
    title('Pendulum bob 2 X and Y acceleration')
    axis tight
end

%%

%Plot pendulum bob speeds
if plot_speed==1
    figure('color',[1 1 1],'name','Double Pendulum bob speeds')
    plot(t,sqrt(xdot1.*xdot1 + ydot1.*ydot1),...
        t,sqrt(xdot2.*xdot2 + ydot2.*ydot2))
    xlabel('Time /s')
    ylabel('Speed /ms^{-1}')
    title('Double Pendulum bob speeds')
    legend('Bob 1','Bob 2')
end

%%

%Plot poincare diagrams. Plot bob angles against bob angular velocities
if plot_poincare==1
    figure('color',[1 1 1],'name','Double Pendulum Poincare Diagram')
    p1=plot(a1,adot1,'go');
    set(p1,'markersize',1,'markerfacecolor',[0,1,0])
    hold on
    p2=plot(a2,adot2,'ko');
    set(p2,'markersize',1,'markerfacecolor',[0,0,0])
    xlabel('Angle /deg')
    ylabel('Angular velocity /deg s^{-1}')
    title('Poincare diagram')
    tl=thelegend({'bob1','bob2'},{'g-','k-'});
    axis equal   
end

%End of code