%RUNGEKUTTA
% Basic numerical solver for sets of coupled first order differential
% equations. Note MATLAB's own solvers, such as ode45, are much more
% sophisticated. In the case of the latter, a Runge-Kutta algorithm is
% used, but an adaptive timestep (or 'x' variable interval) is used.
%
% In this implementation, the set of coupled ODE's are represented by a
% vector equation
%
% dy/dt = f(t,y)
%
% where y is a column vector whose rows are the variables required to be
% integrated. Rather than using an inline function (like ode45), the
% function f(t,y) will be 'hard wired' into this solver. Therefore it will
% need to be adapted to any other ODE set.
%
% LAST UPDATED by Andy French. 10-Dec-2005
%
% Syntax: [t,a1,a2,adot1,adot2]=rungekutta(m1,m2,l1,l2,g,tlim,timestep,...
%    a1_init,a2_init,adot1_init,adot2_init)
%
% m1          - Mass of bob 1 /kg
% m2          - Mass of bob 2 /kg
% l1          - Length of the (rigid) string joining the fixed pivot
%               point to bob 1 /m
% l2          - Length of the (rigid) joining bob2 to bob 1 /m
% g           - Gravitational field strength /ms^-2
% tlim        - Two element vector [tstart,tend] defining start and end 
%               times /s of timescale that dynamics are computed over 
% timestep    - Time interval between outputs /s
% a1_init     - Initial (clockwise) angle of bob 1 measured from the
%               vertical /radians
% a2_init     - Initial (clockwise) angle of bob 2 measured from the
%               vertical radians
% adot1_init  - Initial angular velocity of bob 1 /radians s^-1
% adot2_init  - Initial angular velocity of bob 2 /adians s^-1
%
% t           - Time /s
% a1          - (clockwise) angle of bob 1 measured from the
%               vertical /radians
% a2          - (clockwise) angle of bob 2 measured from the
%               vertical /radians
% adot1       - Angular velocity of bob 1 /radians s^-1
% adot2       - Angular velocity of bob 2 /radians s^-1

function [t,a1,a2,adot1,adot2]=rungekutta(m1,m2,l1,l2,g,tlim,timestep,...
    a1_init,a2_init,adot1_init,adot2_init)

%Define time /seconds
t=tlim(1) : timestep : tlim(2);

%Define initial conditions vector y. Angles must be in radians.
y0=[a1_init;...
    a2_init;...
    adot1_init;...
    adot2_init];

%Set first columns of output vector y to be the input
y=zeros(length(y0),length(t));
y(:,1)=y0(:);

%Evaluate Runge-Kutta parameters and iteratively solve for y
h=timestep;
for n=2:length(t)
    k1=h*f(t(n-1),y(:,n-1),m1,m2,l1,l2,g);
    k2=h*f(t(n-1)+0.5*h,y(:,n-1)+0.5*k1,m1,m2,l1,l2,g);
    k3=h*f(t(n-1)+0.5*h,y(:,n-1)+0.5*k2,m1,m2,l1,l2,g);
    k4=h*f(t(n-1)+h,y(:,n-1)+k3,m1,m2,l1,l2,g);
    y(:,n)=y(:,n-1) + k1/6 + k2/3 + k3/3 + k4/6 ;
end

%Assign outputs. Angles in radians
a1=y(1,:);
a2=y(2,:);
adot1=y(3,:);
adot2=y(4,:);

%%

%Define differential equation function
function dy_by_dt = f(t,y,m1,m2,l1,l2,g)

%Compute angular acceleations
[addot1,addot2]=compute_addot(m1,m2,l1,l2,g,y(1,:),y(2,:),y(3,:),y(4,:));

%Assign outputs
dy_by_dt(1,:)=y(3,:);
dy_by_dt(2,:)=y(4,:);
dy_by_dt(3,:)=addot1;
dy_by_dt(4,:)=addot2;

%End of code

