%SOLVE_DPENDULUM
% Function which solves the dynamics of a double pendulum, i.e. computes
% the time history of the angles between the two rods of the pendulum and
% the vertical. Other dynamic parameters such as kinetic energy and angular
% momentum are also computed.
%
% LAST UPDATED by Andy French. 11-Dec-2005.
%
% Syntax: [t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
%     xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
%     j1,j2,ke1,ke2,pe1,pe2]=solve_dpendulum(m1,m2,l1,l2,g,T,...
%     msolve,timestep,a1_init,a2_init,adot1_init,adot2_init);
%
% m1            - Mass of pendulum bob 1 /kg
% m2            - Mass of pendulum bob 2 /kg
% l1            - Length of the (rigid) string joining the fixed pivot
%                 point to bob 1 /m
% l2            - Length of the (rigid) joining bob2 to bob 1 /m
% g             - Gravitational field strength /ms^-2
% T             - Duration of dynamics /s
% msolve        - Choose solver. ==1 MATLAB's ode45 solver
%                 ==0 4th order Runge-Kutta method
% timestep      - Timestep between solution evaluation
% a1_init       - Initial (clockwise) angle /degrees of bob 1 from vertical
% a2_init       - Initial (clockwise) angle /degrees of bob 2 from vertical
% adot1_init    - Initial angular velocity of bob 1 /degrees per second
% adot2_init    - Initial angular velocity of bob 2 /degrees per second
% 
% t             - Time /s
% a1            - Angle (clockwise) of bob 1 from vertical /degrees
% a2            - Angle (clockwise) of bob 2 from vertical /degrees
% x1            - Cartesian X coordinate of bob 1 /m
% y1            - Cartesian Y coordinate of bob 1 /m
% x2            - Cartesian X coordinate of bob 2 /m
% y2            - Cartesian Y coordinate of bob 2 /m
% adot1         - Angular velocity of bob 1 / deg s^-1
% adot2         - Angular velocity of bob 2 / deg s^-1
% addot1        - Angular acceleration of bob 1 /deg s^-2
% addot2        - Angular acceleration of bob 2 /deg s^-2
% xdot1         - Cartesian X velocity of bob 1 /ms^-1
% ydot1         - Cartesian Y velocity of bob 1 /ms^-1
% xdot2         - Cartesian X velocity of bob 2 /ms^-1
% ydot2         - Cartesian Y velocity of bob 2 /ms^-1
% xddot1        - Cartesian X acceleration of bob 1 /ms^-2
% yddot1        - Cartesian Y acceleration of bob 1 /ms^-2
% xddot2        - Cartesian X acceleration of bob 2 /ms^-2
% yddot2        - Cartesian Y acceleration of bob 2 /ms^-2
% j1            - Angular momentum (Z projection) of bob 1 /kg m^2 s^-1
% j2            - Angular momentum (Z projection) of bob 2 /kg m^2 s^-1
% ke1           - Kinetic energy of bob 1 /Joules
% ke2           - Kinetic energy of bob 2 /Joules
% pe1           - Gravitational potential energy of bob 1 /Joules
% pe2           - Gravitational potential energy of bob 2 /Joules

function [t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2]=solve_dpendulum(m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init);

%Initliase outputs
t=[];
a1=[];
a2=[];
adot1=[];
adot2=[];

%Convert initial angles and angular velocities to radian units
a1_init=a1_init*pi/180;
a2_init=a2_init*pi/180;
adot1_init=adot1_init*pi/180;
adot2_init=adot2_init*pi/180;

%Compute total energy (which is conserved)
[E_tot,A,B,C]=total_energy(a1_init,a2_init,...
    adot1_init,adot2_init,m1,m2,l1,l2,g);

%Solve in TT second steps
TT=100;

%Determine number of TT intervals
num_TT=ceil(T/TT);

for n=1:num_TT

    %Display message to command window
    disp(['Solver interation ',num2str(n),' \',num2str(num_TT)])

    %Determine start time
    tstart=(n-1)*TT;

    %Determine end time
    if n~=num_TT
        tend=n*TT;
    else
        tend=T;
    end

    %Assign initial conditions
    if n==1
        a1INIT=a1_init;
        a2INIT=a2_init;
        adot1INIT=adot1_init;
        adot2INIT=adot2_init;
    else
        a1INIT=a1(end);
        a2INIT=a2(end);
        adot1INIT=adot1(end);
        adot2INIT=adot2(end);
    end

    %Determine angles and angular velocities
    if msolve==1
        %Use MATLAB ODE solver

        %Define initialisation vector z0
        z0(1)=a1INIT;
        z0(2)=a2INIT;
        z0(3)=adot1INIT;
        z0(4)=adot2INIT;

        %Call solver ode45 using inline function dpendulum_func defined below
        [tt,z]=ode113(@dpendulum_func,[tstart,tend],z0.',[],m1,m2,l1,l2,g,E_tot);

        %Interpolate to yield results equispaced by timestep
        tt_equispace=tstart:timestep:tend;  
        zz=zeros(length(tt_equispace),4);
        for k=1:4
            zz(:,k)=interp1(tt.',z(1:end,k),tt_equispace.');
        end
        tt=interp1(tt,tt,tt_equispace.');

        %Assign output vectors (angles in radians still)
        a1=[a1,zz(2:end,1).'];
        a2=[a2,zz(2:end,2).'];
        adot1=[adot1,zz(2:end,3).'];
        adot2=[adot2,zz(2:end,4).'];
        t=[t,tt(2:end).'];

    else
        %Use (supplied) Runge-Kutta solver to
        %determine angles /radians and angular velocities /radians per second
        [tt,aa1,aa2,aadot1,aadot2]=rungekutta(m1,m2,l1,l2,g,[tstart,tend],timestep,...
            a1INIT,a2INIT,adot1INIT,adot2INIT);
        t=[t,tt(2:end)];
        a1=[a1,aa1(2:end)];
        a2=[a2,aa2(2:end)];
        adot1=[adot1,aadot1(2:end)];
        adot2=[adot2,aadot2(2:end)];
    end
end

%Compute Cartesian X and Y coordinates /metres. Origin is set at the fixed point
%from which the pendulum hangs
x1=l1*sin(a1);
y1=-l1*cos(a1);
x2=x1 + l2*sin(a2);
y2=y1 - l2*cos(a2);

%Compute Cartesian velocities /metres per second
xdot1=l1*cos(a1).*adot1;
ydot1=l1*sin(a1).*adot1;
xdot2=xdot1 +l2*cos(a2).*adot2;
ydot2=ydot1 + l2*sin(a2).*adot2;

%Compute angular accelerations /radians per second per second
[addot1,addot2]=compute_addot(m1,m2,l1,l2,g,a1,a2,adot1,adot2);

%Compute Cartesian accelerations /metres per second per second
xddot1=l1*( cos(a1).*addot1 - sin(a1).*adot1.*adot1);
yddot1=l1*( sin(a1).*addot1 + cos(a1).*adot1.*adot1);
xddot2=xddot1 + l2*( cos(a2).*addot2 - sin(a2).*adot2.*adot2);
yddot2=yddot1 + l2*( sin(a1).*addot2 + cos(a2).*adot2.*adot2);

%Compute angular momenta /kg * m^2 * s^-1
%projected onto the Cartesian Z direction
j1=m1*dot([zeros(size(x1));zeros(size(x1));ones(size(x1))],...
    cross([x1;y1;zeros(size(x1))],[xdot1;ydot1;zeros(size(x1))],1),1);
j2=m2*dot([zeros(size(x1));zeros(size(x1));ones(size(x1))],...
    cross([x2;y2;zeros(size(x1))],[xdot2;ydot2;zeros(size(x1))],1),1);

%Compute Kinetic and Gravitational Potential energies /Joules
ke1=0.5*m1*( xdot1.*xdot1 + ydot1.*ydot1 );
ke2=0.5*m2*( xdot2.*xdot2 + ydot2.*ydot2 );
pe1=m1*g*y1;
pe2=m2*g*y2;

%Assign angular outputs in degrees, with angles wrapped onto [-180,180]
a1=wrap_angle(a1)*180/pi;
a2=wrap_angle(a2)*180/pi;
adot1=adot1*180/pi;
adot2=adot2*180/pi;
addot1=addot1*180/pi;
addot2=addot2*180/pi;

%%

%Wrap angle to [-pi,pi]
function theta_wrap=wrap_angle(theta)
x=cos(theta);
y=sin(theta);
theta_wrap=atan2(y,x);

%%

%Inline function for use with solver
function dz_by_dt=dpendulum_func(t,z,m1,m2,l1,l2,g,E)

%Assign elements of z
a1=z(1);
a2=z(2);
adot1=z(3);
adot2=z(4);

%Compute angular accelerations /radians per second per second
[addot1,addot2]=compute_addot(m1,m2,l1,l2,g,z(1),z(2),z(3),z(4));

% %Compute angular velocity adot2 using conservation of energy
% [E,A,B,C]=total_energy(a1,a2,adot1,adot2,m1,m2,l1,l2,g);
% adot2=(-C + sqrt(C*C -4*B*(A-E)))/(2*B);

%Initialise and assign outputs
dz_by_dt=zeros(4,1);
dz_by_dt(1)=adot1;
dz_by_dt(2)=adot2;
dz_by_dt(3)=addot1;
dz_by_dt(4)=addot2;

%End of code

