%DPENDULUM
% Script which computes the dynamics of a double pendulum given 
% defined initial conditions. Dynamical variables a1, a2 are the 
% angles of each pendulum bob's rod measured anticlockwise from the
% vertical.
%
% LAST UPDATED by Andy French. 10th December 2005.

%%% INPUT PARAMETERS %%%

%Bob masses /kg
m1=1;
m2=4;

%Rigid connecting rod lengths (negligible mass) /m
l1=3;
l2=1;

%Gravitational acceleration /ms^-2
g=9.81;

%Duration of solution set /s  (10 frames per second is not a bad idea)
T=20;

%Solver flag. If msolve=1, use MATLAB's built in ode45 ODE solver. If =0,
%use rungekutta.m using the time-step defined in 'timestep'
msolve=1;
timestep=0.01;

%%

%%% INITIAL CONDITIONS %%%

%Initial bob rod angles in degrees. (Measured anticlockise from the
%vertical).
a1_init=160;
a2_init=160;

%Initial bob angular velocities /degrees per second
adot1_init=-180;
adot2_init=360;

%%

%%% OUTPUT LIST %%%

%Plot bob rod angles vs time
plot_a=0;

%Plot bob X and Y values vs time
plot_xyt=0;

%Plot X,Y trajectory
plot_xy=1;

%Generate N PNG files for generation of an animated GIF
%This includes a representation of the pendulum using shaded spheres!
plot_png=1;
N=200;

%Plot Kinetic and Potential energies vs time
plot_ke_pe=1;

%Plot angular momenta vs time (Note since motion is defined to be planar, this will
%be a scalar rather than vector quantity - the actual direction is the Z
%Cartesian axis)
plot_j=0;

%Plot bob angular velocity and angular acceleration vs time
plot_adot_addot=0;

%Plot Cartesian X and Y velocities and X and Y trajectories vs time
plot_xydot_xyddot=0;

%Plot bob speeds vs time
plot_speed=0;

%Draw 'Poincare diagram' of bob angle vs angular acceleration, for each bob
plot_poincare=1;

%%

%%% SOLVE! %%%

%Add \Functions directory to MATLAB path
addpath([pwd,'\Functions'])

%Compute bob angles a1 and a2 in degrees for times within [0,T] as well as
%bob Cartesian X,Y positions, angular velocities / degrees per second,
%angular acceleration /degrees per second per second
%Cartesian X and Y velocities, Cartesian X and Y accelerations, Angular
%momentum, Kinetic Energy and Gravitational Potential Energy.
[t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2]=solve_dpendulum(m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init);

%%

%%% SAVE & PLOT RESULTS %%%

%Save data to a .mat file
save 'double_pendulum';

%Plot graphs
plot_dpendulum(plot_a,plot_xyt,plot_xy,plot_png,plot_ke_pe,...
    plot_j,plot_adot_addot,plot_xydot_xyddot,plot_speed,plot_poincare,...
    t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2,...
    m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init,N);

%Remove \Functions directory from MATLAB path
rmpath([pwd,'\Functions'])

%End of code
